#!/usr/bin/python
# -*- coding: utf_8 -*-
import shutil
import time
from os.path import exists, dirname, isdir, isfile
from sys import argv
import dbus
import ip
import gtk
import os
import re
import subprocess
from excepciones import ServiceException, XDMCPServiceException
#ltsp-conf installation path
LTSPCONF_PATH = '/usr/lib/ltspconfig'

#Paths de configuraciones
DEFAULT_TFTPD_HPA='/etc/default/tftpd-hpa'
CONF_SUPER_SERVER='/etc/inetd.conf'
CONF_POSTINST_TFTP='/var/lib/dpkg/info/tftpd-hpa.postinst'
DHCPD_CONF = '/etc/dhcp/dhcpd.conf'
TMP_DHCPD_CONF = '/tmp/tmp_dhcpd.conf'
DIFF = '/usr/bin/diff'
NETWORKING = '/etc/init.d/networking'
DEFAULT_INT_DHCP = '/etc/default/isc-dhcp-server'
TMP_DEFAULT_INT_DHCP = '/tmp/tmp_dhcp3-server'
LTS_CONF = '/opt/ltsp/i386/etc/lts.conf'
NFS_CONF = '/etc/exports'
TMP_NFS_CONF = '/tmp/tmp_exports'
ALSA_CONF = '/etc/asound.conf'
PULSE_DEFAULT_CONF = '/etc/default/pulseaudio'
#Texto de configuraciones
PULSE_SETTINGS = 'PULSEAUDIO_SYSTEM_START=0\nDISALLOW_MODULE_LOADING=1\n'
ALSA_SETTINGS = 'pcm.!default {\n\ttype pulse\n}\nctl.!default {\n\ttype pulse\n}\n'
#IPTABLES_ACTIVE tiene el path del archivo donde estan las reglas de configuración de iptables
IPTABLES_ACTIVE='/etc/iptables/rules'
TMP_IPTABLES_ACTIVE = '/tmp/tmp_active'
#IPTABLES_SCRIPT tiene el script que se ejecuta para levantar las iptables
IPTABLES_SCRIPT = LTSPCONF_PATH + '/iptables.sh'
#IPTABLES es el script personalizado por le grupo Lihuen para nuestra distribución, si se usa en otra Distribución como debian se debe
#guardar en forma diferente las reglas de iptables, por lo tanto preguntamos si existe nuestro script para saber como debemos 
IPTABLES = "/etc/init.d/iptables-lihuen"
#Path del archivo de configuración de iptables guardado
DEBIAN_IPTABLES_CONF = "/etc/firewall.conf"
#iptables_save guarda la ocnfiguracion del momento
IPTABLES_SAVE = "/sbin/iptables-save"
#iptables_restore restaura la configuración de las reglas de iptables
ITPABLES_RESTORE = "/sbin/iptables-restore"
#GDM_CONF es el archivo de configuración de gdm utilizado para verificar si está habilitado para usar xmdcp
GDM_CONF = "/etc/gdm3/daemon.conf"
TMP_GDM_CONF="/tmp/tmp_gdm.conf"
#Directorio compartido para reportar las mac de los hosts
COMPARTIDO ="/opt/compartido/"
#Paths de scripts de arranque
path_servicios = {'NETWORK_MANAGER': '/etc/init.d/network-manager', 'NETWORKING': '/etc/init.d/networking', 'PULSE_INIT': '/etc/init.d/pulseaudio','DHCPD_INIT':'/etc/init.d/isc-dhcp-server', 'NFS_INIT':'/etc/init.d/nfs-kernel-server','OPENBSDINETD_INIT':'/etc/init.d/openbsd-inetd'}
servicios_conexion = {
	True:('22', 't', 'Hay un error en la configuración del servicio SSH'),
	False:('177', 'u', 'Reiniciar gdm con el comando: /etc/init.d/gdm3 restart')}
	
servicios={'67':('u',),'69':('u',),'111':('t','u'),'2049':('t','u')}
nombre_servicio={'67':'DHCP','69':'TFTP','111':'PORTMAP','2049':'NFS'}

PUERTO_RE = re.compile('([0-9]+)$')

def reiniciar_servicio(path_script):
    "Dado un script de arranque de un servicio invoca: 'path_script restart'"
    os.system(path_script + ' restart')

def verificar_servicios (ssh):
	errores_servicios=''
	serv,proto,descripcion = servicios_conexion[ssh]
	XDMCPError = False
	if not verificar_puerto(serv,proto):
		errores_servicios = errores_servicios + descripcion
		if not ssh:
			XDMCPError = True # Nos interesa si sólo falla XDMCP
	for servicio  in servicios:
		for protocolo in servicios[servicio]:
			if not verificar_puerto(servicio,protocolo):
				XDMCPError = False
				errores_servicios = errores_servicios + 'Hay un error en la configuración del servicio '+ nombre_servicio[servicio] + '\n'
	if XDMCPError:
		raise XDMCPServiceException(errores_servicios)
	if errores_servicios != '':
		raise ServiceException(errores_servicios)
	

def verificar_puerto(puerto, protocolo):
    netstat = subprocess.Popen('netstat -' + protocolo + 'anp', stdout=subprocess.PIPE, shell=True)
    netstat.wait()
    netstat.stdout.readline()
    netstat.stdout.readline()
    for line in netstat.stdout:
        direccion_listen = line.split()[3]
        if PUERTO_RE.search(direccion_listen).group(1) == puerto:
            return True
    return False

def hacer_backup(path):
    """Intenta crear un backup de path si lo puede crear o si no existe el original retorna True
si path existe pero no puede hacer el backup retorna False"""
    if not (exists(path)):
        #si no existe el path no necesita hacer backup y sale del método
        return True 
    src = path
    dst = path + str(int(time.time()))
    try:    
        shutil.copy2(src,dst)
    except:
        return False
    return True

def guardar_cambios_en_archivos(orig, temporal):
    """retorna false si son iguales, true en caso contrario, copiamos los archivos y retorna true
     porque tiene que reiniciar los servicios"""
    if not subprocess.call(DIFF + ' '+ temporal + ' '+orig, shell=True, stdout=subprocess.PIPE) ==0 :
        if hacer_backup(str(orig)):
            try:    
                shutil.copy2(temporal, orig)
            except:
                return False
        return True 
    return False

def configurar_sonido():
    """Configura por defecto el uso de PULSE en el servidor LTSP"""
    alsa = ConfFile(ALSA_CONF, 'w')
    alsa.write(ALSA_SETTINGS)
    alsa.close()
    alsa.limpiarBackups()
    pulse = ConfFile(PULSE_DEFAULT_CONF, 'w')
    pulse.write(PULSE_SETTINGS)
    pulse.close()
    pulse.limpiarBackups()   

        
def error(mensaje):
    """Muestra un dialogo tipo warning con el mensaje pasado como argumento"""
    a = gtk.MessageDialog(None,0, gtk.MESSAGE_WARNING,gtk.BUTTONS_OK,mensaje)
    a.set_title('Ltspconfig: Error')
    a.run()
    a.destroy()

def interfaces():
    """Si puede Retorna una colección de interfaces de red de acuerdo a la intefaz
org.freedesktop.SystemToolsBackends, en caso de fallar muestra un error y retorna False"""
    system_bus = dbus.SystemBus()
    proxy = system_bus.get_object('org.freedesktop.SystemToolsBackends', '/org/freedesktop/SystemToolsBackends/IfacesConfig')
    try:
        iface = proxy.get(dbus_interface='org.freedesktop.SystemToolsBackends')         
        return iface
    except dbus.exceptions.DBusException, e:
            error('Error en DBUS, debe ser root para configurar LTSP')              
    return False
    
def listaPlacas():
    """Retorna una lista con los nombres de las placas ethernet del sistema"""
    lista = []
    iface = interfaces()
    for i in iface[0]:
        #print i
        lista.append(i[0])  
    return lista

def listaDriversXorg():
    """Retorna la lista de drivers para placas de vídeo soportada por la imagen de
    los clientes, en caso que el directorio donde se buscan no exista devuelve una
    lista vacía"""
    drivers_list = []
    XORG_DRIVERS = '/opt/ltsp/i386/usr/lib/xorg/modules/drivers/'
    if isdir(XORG_DRIVERS):
        files = os.listdir(XORG_DRIVERS)
        for each in files:
            if isfile(XORG_DRIVERS + each):
                # Entra tanto si es un archivo, como si es un
                # link simbólico a un archivo
                limite = each.find('_drv.so')
                if limite != -1: 
                    # Removemos la parte del nombre que no
                    # nos interesa
                    drivers_list.append(each[0:limite])
        drivers_list.sort()
    return drivers_list

    
class Interfaz:
    """Clase para comunicarse con SystemToolsBackends vía dbus, al crear una instancia de esta clase
intenta conectarse y obtener los datos de la interfaz seleccionada"""
    def __init__(self, nombre_interfaz):
        """Dado un string con un nombre de interfaz pide sus datos (entre otros) a SystemToolsBackends
y guarda en variables de instancia"""
        nombre_interfaz = dbus.String(nombre_interfaz)
        i = 0
        self.iface = interfaces()
        while (self.iface[0][i][0] != nombre_interfaz):
            i+=1            
        self.indice = i

    def obtener_ip(self):
        "Retorna la ip de la interfaz seleccionada"
        return self.iface[0][self.indice][4] if self.iface[0][self.indice][9] == 'static' else False    
    def configurar_ip (self,dir_ip,mascara,gateway):
        mascara_int = ip.stringToIp(mascara)
        ip_int = ip.stringToIp(dir_ip)
        net_int = ip.getNet(ip_int, mascara_int)
        wildcard = ip.ipToString(ip.wildcard(mascara_int))
        broadcast = ip.ipToString(ip.netBroadCast(net_int, mascara_int))
        self.iface[0][self.indice]=dbus.Struct((self.iface[0][self.indice][0],dbus.Int32(1), dbus.Int32(0), dbus.Int32(1), dbus.String(broadcast), dbus.String(mascara), dbus.String(dir_ip), dbus.String(wildcard), dbus.String(gateway), dbus.String(u'static')))
        #self.proxy.set(self.iface,dbus_interface='org.freedesktop.SystemToolsBackends')
        print self.iface
        
    def obtener_mascara(self):
        "Retorna la máscara de la ip de la interfaz"
        return self.iface[0][self.indice][5]
    
    def obtener_gateway(self):
        "Retorna el gateway configurado para la interfaz"
        return self.iface[0][self.indice][8]
    
    def obtener_nombre_interfaz(self):
        "Retorna el nombre de la interfaz"
        return self.iface[0][self.indice][0]
    

class ConfFile(file):
    """Clase que desciende de file, realiza las mismas operaciones pero además hace backups de los originales
    además provee un método que permite borrar el último backup en caso que no sea necesario"""
    def __init__(self, path, mode='r'):
        if exists(path):
            self.backupPath = path + str(int(time.time()))
            shutil.copy2(path, self.backupPath)
        else:
            self.backupPath = None
        file.__init__(self, path, mode)
    
    def limpiarBackups(self):
        """"retorna false si el archivo no cambio respecto a su versión original, en caso contrario retorna true,
        esto indica que puede ser necesario reiniciar algún servicio"""
        if self.backupPath == None:
            return True
        
        if subprocess.call(DIFF + ' ' + self.backupPath + ' ' + self.name, shell=True, stdout=subprocess.PIPE) ==0 :
            os.remove(self.backupPath)
            return False
        return True
