#!/usr/bin/python
# -*- coding: utf_8 -*-
"""Módulo para validar direcciones IPv4"""
import re
import math
from excepciones import LTSPConfigException
ipRe = re.compile('^([0-9]{1,3}\.){3}[0-9]{1,3}$')
maxRep = math.pow(2, 32)

class IPException(LTSPConfigException):
	def __init__(self, description):
		self.value = description
	def __repr__(self):
		return repr(self.value)

def stringToIp(string):
	""" Dada una IP de IPv4 en un string con notación decimal separada por puntos
	retorna la retorna en un número entero utilizable en el resto de las rutinas
	de este módulo"""
	global ipRe
	if ipRe.match(string) == None:
		raise IPException('Error de formato en la ip: ' + string)
	octetos = string.split('.')
	ip32bits = 0
	desplazamiento = 1
	for octeto in octetos:
		octeto = int(octeto)
		if octeto > 255:
			raise IPException('Octeto mayor a 255 en: ' + string)
		ip32bits = ip32bits + octeto * desplazamiento
		desplazamiento *= 256

	return ip32bits

def getNet(ip, mask):
	"""Dada ip y mask en decimal retorna un entero con la red a la que pertenece"""
	return ip & mask

def ipInRange(net, mask, ip):
	"""Dada net, mask e ip en decimal retorna True si ip pertenece a la red net"""
	if net == ip & mask:
		return True
	return False

def ipsAreInTheSameNet(ip1, ip2, mask):
	"""Dadas 2 ip y una máscara retorna true si las ip están en la misma red"""
	if ip1 & mask == ip2 & mask:
		return True
	return False

def netBroadCast(net, mask):
	"""Dada una red y una máscara retorna la dirección de broadcast"""
	global maxRep
	return net | int(~mask % maxRep)

def ipToString(ip):
	"""Dada una ip en decimal retorna un string con la ip en notación decimal
	separada por puntos"""
	string = ''
	for i in range(0, 4):
		string += str(ip % 256) + '.'
		ip /= 256
	return string[:-1]

def wildcard(mask):
	"Dada una máscara retorna el resultado de invertir los bits de la misma"
	return int(~mask % maxRep) 
	
	
