#!/usr/bin/python
# -*- coding: utf_8 -*-
import gtk
import gtk.glade
import lts_conf_global
import os
import os.path
import re
import gobject
import ConfigParser
from configuraciones import error, LTSPCONF_PATH, COMPARTIDO

"""Contiene clases para la configuración individual de cada cliente en lts.conf"""

isKeymap = re.compile('([^\.]+)\.kmap(\.gz)?') # Expresión regular que matchea con: *.kmap[.gz]

def detectar_keymaps(root):
	"Detecta todos los keymaps instalados en el entorno de los clientes"
	global isKeymap
	files = os.listdir(root + '/usr/share/keymaps/i386/qwerty/')
	keymaps = []
	for each in files:
		match = isKeymap.match(each)
		if match != None:
			keymaps.append(match.group(1))
	keymaps.sort()
	return keymaps

class LtsClientes:
	"Dibuja el diálogo para la configuración individual de los clientes"
	def __init__(self, parser):
		
		self.parser = parser
		
		self.wTree = gtk.glade.XML(LTSPCONF_PATH + '/ltsp-clientes.glade')
		self.wTree.signal_autoconnect(self)
		traer = self.wTree.get_widget
		self.ventana = self.wTree.get_widget('configuracionDeCliente')		
		self.equipo = self.wTree.get_widget('equipo')
		cell = gtk.CellRendererText()
		self.equipo.pack_start(cell, True)
		self.equipo.add_attribute(cell, 'text', 0) 
		self.equipoActivo = -1
		self.agregar_cliente = traer('agregarCliente')
		self.boton_quitar = traer('quitar')
		self.widgets={'XSERVER':ComboAdapter(traer('placa_video')),
				 'X_MODE_0':ComboAdapter(traer('resolucion')),
				 'X_COLOR_DEPTH':ComboAdapter(traer('profundidad_color')),
				 'X_VERTREFRESH':TextAdapter(traer('vert_refresh')),
				 'X_HORZSYNC':TextAdapter(traer('hor_refresh')),
				 'X_MOUSE_DEVICE':TextAdapter(traer('mouse')),
				 'X_MOUSE_PROTOCOL':ComboAdapter(traer('driver_mouse')),
				 'XKBLAYOUT':ComboAdapter(traer('distribucion_teclado'))
				 }
		for opcion in self.widgets:
			self.widgets[opcion].llenar(opcion)
		self.cargar_clientes()		
		self.ventana.run()


	def cerrar(self, widget, data1=None, data2=None):
		"""Cierra la ventana de configuraciones del cliente guardando las configuraciones del cliente seleccionado
en la estructura del IniParser"""
		self.guardar_campos_cliente(self.equipo.get_active_text())## Revisar cuando implementemos lo de nombre
		self.ventana.destroy()
	
	def cargar_clientes(self):
		"""Carga el combobox de equipos con los datos almacenados en el IniParser, es invocado desde __init__,
agregar y quitar"""
		equipos = gtk.ListStore(gobject.TYPE_STRING)
		self.equipo.set_model(equipos)
		lista_equipos = self.parser.all_sections()
		for seccion in lista_equipos:			
			self.equipo.append_text(seccion)		
		self.equipo.set_active(0) # Desencadena el evento que invoca a self.cambiar_equipo
		self.equipo.show_all()	
		
	def cambiar_equipo(self, widget):
		"Guarda los datos del cliente anteriormente configurado y muestra los datos del cliente que se seleccionó ahora"
		active = self.equipo.get_active()
		modelo = self.equipo.get_model() 
		if self.equipoActivo >=0:
			self.guardar_campos_cliente(modelo[self.equipoActivo][0]) # Guarda el equipo anterior
		self.mostrar_campos_cliente(modelo[active][0])
		self.equipoActivo = active
		if modelo[self.equipoActivo][0].lower() == 'default':
			self.boton_quitar.set_sensitive(False)
			return
		self.boton_quitar.set_sensitive(True)
	
	def mostrar_campos_cliente(self, seccion):
		"Para una sección dada carga los datos del cliente en los combobox y textbox"
		for opcion in self.widgets:
			self.widgets[opcion].set(self.parser.get(seccion,opcion))
	
	def guardar_campos_cliente(self, seccion):
		"""Dada una sección guarda todas sus opciones en la estructura del IniParser, si la opción
está vacía en el diálogo pero existe en el IniParser se la borra de este último"""
		for opcion in self.widgets:
			if self.widgets[opcion].get() !=  '':
				self.parser.set(seccion, opcion, self.widgets[opcion].get())
			elif self.parser.has_option(seccion,opcion):
				self.parser.remove_option(seccion,opcion)
		
	def agregar(self, widget, data=None):
		"Agrega o modifica un equipo"
		# Falta programar la acción de modificar, quizá sea mejor dividir esto en 2 métodos uno para agregar
		# y otro para modificar eso ahorraría muchas desprolijidades

		# Preguntamos el nombre del botón para saber que hacer (horrible!!!!!)
		if widget.get_name() == 'agregar':
			self.wTree.get_widget('nombre').set_text('')
			self.wTree.get_widget('mac').set_text('')
		else:			
			pass # Editar carga los valores anteriores
		self.agregar_cliente.set_transient_for(self.ventana) # Lo forzamos para que esté on top
		#busca hosts que fueron reportados con errores en /opt/compartido
		self.reportados = self.wTree.get_widget('elegir_reportado')
		if not (os.path.exists(COMPARTIDO)):
			error('Se creará el directorio /opt/compartido para utilizar en caso de necesitar personalizar algún campo de un cliente')
			os.mkdir(COMPARTIDO)
		self.reportados.set_current_folder(COMPARTIDO)
		self.reportados.set_filename(COMPARTIDO)
		self.reportados.set_width_chars(15)
		self.reportados.connect("file-set",self.obtener_reporte)
		self.agregar_cliente.run()
		self.wTree.get_widget('mac').get_text()
		mac = self.wTree.get_widget('mac').get_text()
		if mac:
			try:
				self.parser.add_section(mac)
			except ConfigParser.DuplicateSectionError:
				#modificar el nombre por el que se ingreso como nombre del host
				pass
		self.cargar_clientes()
		# Ponemos en active el equipo recién agregado, Feo feo
		index = 0
		for each in self.equipo.get_model():
			if each[0] == mac:
				self.equipo.set_active(index)
				self.mostrar_campos_cliente(mac)
			index += 1

	def obtener_reporte(self,widget,data=None):
		"Obtiene la mac del archivo reportar"
		archivo = widget.get_filename()
		if	archivo and not os.path.isdir(archivo):
			nombre = os.path.basename(archivo)
			f = open(archivo)
			try:
				self.wTree.get_widget('mac').set_text(f.readline().strip())
				self.wTree.get_widget('nombre').set_text(nombre)
			finally:
				f.close()
		
	def agregarClienteCerrar(self, widget, data1=None, data2=None):
		"Cierra el dialogo para agregar y modificar clientes"
		self.agregar_cliente.hide()
		return True # Para que no envíe destroy a la ventana
		
	def quitar(self,widget, data=None):
		equipo = self.equipo.get_active_text()
		# Ahora al estar deshabilitado el botón en la sección default no hace falta el chequeo
		#if equipo.lower() == 'default':
		#	error('La sección Default no debe eliminarse :-)')
		#else:
		self.parser.remove_section(equipo)
		self.equipoActivo=-1
		self.cargar_clientes()
			

		
class Adapter:
	def __init__(self,adaptee):
		self.adaptee = adaptee
	
	def llenar(self, key):
		pass
		
	def set(self, text):
		pass
	
	def get(self):
		pass

		
class ComboAdapter(Adapter):
		
	def llenar(self, key):
		"""Muestra los datos de un combo box recibe una clave y carga una lista
de alternativas para una opción de lts.conf. Los parámetros posibles están en el módulo lts_conf_global.py."""
		self.adaptee.set_model(gtk.ListStore(str))
		cell = gtk.CellRendererText()
		self.adaptee.pack_start(cell, True)
		self.adaptee.add_attribute(cell, 'text', 0)
		self.adaptee.append_text('Automático')
		for each in lts_conf_global.multiopcion[key]:
			self.adaptee.append_text(each)
		self.adaptee.set_active(0)
		
	def set(self, text):
		index = -1
		count = 0
		for each in self.adaptee.get_model():
			if each[0] == text:
				index = count
			count += 1
		self.adaptee.set_active(index if index >= 0 else 0)
	
	def get(self):		
		if self.adaptee.get_active() == 0:
			return ''
		return self.adaptee.get_active_text()
	
	
class TextAdapter(Adapter):
	
	def llenar(self,key):
		self.adaptee.set_text('')
		
	def set(self,text):
		self.adaptee.set_text(text)
		
	def get(self):
		return self.adaptee.get_text()
